
/*
 * Copyright © 2001 Novell, Inc. All Rights Reserved.
 *
 * You may distribute under the terms of either the GNU General Public
 * License or the Artistic License, as specified in the README file.
 *
 */

/*
 * FILENAME		:	nwplglob.h
 * DESCRIPTION	:	Perl globbing support for NetWare.
 * Author		:	HYAK
 * Date			:	January 2001.
 *
 */


#ifndef __NWplGlob_H__
#define __NWplGlob_H__


void fnDoPerlGlob(char** argv, char* fileName);


#endif	// __NWplGlob_H__

