+{
   locale_version => 1.14,
# eng doesn't require tailoring
   entry => <<'ENTRY', # for DUCET v7.0.0
00E0      ; [.190D.0020.0002] # LATIN SMALL LETTER A WITH GRAVE
0061 0300 ; [.190D.0020.0002] # LATIN SMALL LETTER A WITH GRAVE
0061 0340 ; [.190D.0020.0002] # LATIN SMALL LETTER A WITH GRAVE
00C0      ; [.190D.0020.0008] # LATIN CAPITAL LETTER A WITH GRAVE
0041 0300 ; [.190D.0020.0008] # LATIN CAPITAL LETTER A WITH GRAVE
0041 0340 ; [.190D.0020.0008] # LATIN CAPITAL LETTER A WITH GRAVE
00E9      ; [.196F.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0301 ; [.196F.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0341 ; [.196F.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
00C9      ; [.196F.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0301 ; [.196F.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0341 ; [.196F.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
00EB      ; [.1970.0020.0002] # LATIN SMALL LETTER E WITH DIAERESIS
0065 0308 ; [.1970.0020.0002] # LATIN SMALL LETTER E WITH DIAERESIS
00CB      ; [.1970.0020.0008] # LATIN CAPITAL LETTER E WITH DIAERESIS
0045 0308 ; [.1970.0020.0008] # LATIN CAPITAL LETTER E WITH DIAERESIS
00F1      ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
00F3      ; [.1AA2.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1AA2.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1AA2.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
00D3      ; [.1AA2.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1AA2.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1AA2.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
ENTRY
};
